==> osam_flutter-3.3.5.tar.gz: lib/src/navigation/osam_navigator.dart <==
import 'package:after_layout/after_layout.dart';
import 'package:animations/animations.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:osam_flutter/src/navigation/osam_route.dart';
import '../../osam_flutter.dart';

typedef OsamDialog = Future Function(BuildContext context);

class OsamNavigator<P extends NavigationPresenter> extends StatefulWidget {
  final Map<Type, WidgetBuilder> routes;
  final Map<Type, OsamDialog> dialogs;
  final bool isMaterial;
  final bool log;
  final bool automaticHideKeyBoardOnPush;

  const OsamNavigator(
      {Key key,
      this.routes,
      this.isMaterial = false,
      this.log = false,
      this.automaticHideKeyBoardOnPush = false,
      this.dialogs})
      : super(key: key);

  @override
  _OsamNavigatorState createState() => _OsamNavigatorState<P>();
}

class _OsamNavigatorState<P extends NavigationPresenter>
    extends State<OsamNavigator<P>> with AfterLayoutMixin {
  final _routes = <OsamRoute<Object>>[];
  var lastKnownStackSize = 0;

  Map<Type, WidgetBuilder> get routes => widget.routes;

  Map<Type, OsamDialog> get dialogs => widget.dialogs;

  @override
  Widget build(BuildContext context) =>
      routes[context.presenter<P>().firstRoute.runtimeType](context);

  void silentPop(P presenter) {
    if (_routes.length > 1) {
      lastKnownStackSize = lastKnownStackSize - 1;
      if (_routes.length > lastKnownStackSize) {
        _routes.removeLast();
        presenter.pop();
      }
    }
  }

  void hideKeyBoard() =>
      SystemChannels.textInput.invokeMethod('TextInput.hide');

  PageRoute _pageRoute({WidgetBuilder widgetBuilder, RouteSettings settings}) =>
      widget.isMaterial
          ? MaterialPageRoute(builder: widgetBuilder, settings: settings)
          : CupertinoPageRoute(builder: widgetBuilder, settings: settings);

  @override
  void afterFirstLayout(BuildContext context) {
    final navigator = Navigator.of(context);
    final currentContext = navigator.overlay.context;
    final presenter = context.presenter<P>();
    final initialRoutes = presenter.currentRoutes;
    _routes.addAll(initialRoutes);
    if (initialRoutes.length > 1) {
      final others = initialRoutes.sublist(1, initialRoutes.length);
      others.forEach((route) {
        final target = routes[route.runtimeType] ?? dialogs[route.runtimeType];
        if (!(target is WidgetBuilder)) {
          (target(currentContext) as Future).then((_) {
            route.complete();
            silentPop(presenter);
          });
        } else {
          navigator.push(_pageRoute(widgetBuilder: target)).then((value) {
            route.complete();
            silentPop(presenter);
          });
        }
      });
      if (widget.automaticHideKeyBoardOnPush) hideKeyBoard();
    }
    lastKnownStackSize = _routes.length;
    presenter.navigation.routes.addListener(() {
      final newRoutes = presenter.currentRoutes;
      if (widget.log) {
        print('Last routes: $_routes');
        print('New routes: $newRoutes');
      }
      final routesDifferentNumber = newRoutes.length - _routes.length;
      if (routesDifferentNumber == 0 && newRoutes.last != _routes.last) {
        final newRoute = newRoutes.last;
        final target =
            routes[newRoute.runtimeType] ?? dialogs[newRoute.runtimeType];
        if (!(target is WidgetBuilder)) {
          (target(currentContext) as Future).then((_) {
            newRoute.complete();
            silentPop(presenter);
          });
        } else {
          navigator
              .pushReplacement(_pageRoute(widgetBuilder: target))
              .then((value) {
            newRoute.complete();
            silentPop(presenter);
          });
        }
        lastKnownStackSize = lastKnownStackSize + 1;
        if (widget.automaticHideKeyBoardOnPush) hideKeyBoard();
      } else if (routesDifferentNumber > 0) {
        final pushRoutes = newRoutes.sublist(_routes.length, newRoutes.length);
        pushRoutes.forEach((route) {
          final target =
              routes[route.runtimeType] ?? dialogs[route.runtimeType];
          if (!(target is WidgetBuilder)) {
            (target(currentContext) as Future).then((_) {
              route.complete();
              silentPop(presenter);
            });
          } else {
            navigator.push(_pageRoute(widgetBuilder: target)).then((_) {
              route.complete();
              silentPop(presenter);
            });
          }
        });
        if (widget.automaticHideKeyBoardOnPush) hideKeyBoard();
        lastKnownStackSize = newRoutes.length;
      } else if (routesDifferentNumber < 0) {
        if (_routes.first != newRoutes.first) {
          lastKnownStackSize = newRoutes.length;
          final newRoute = newRoutes.first;
          final target =
              routes[newRoute.runtimeType] ?? dialogs[newRoute.runtimeType];
          if (!(target is WidgetBuilder)) throw Error();
          navigator
              .pushAndRemoveUntil(
                  _pageRoute(widgetBuilder: target), (route) => false)
              .then((value) {
            newRoute.complete();
            silentPop(presenter);
          });
          if (widget.automaticHideKeyBoardOnPush) hideKeyBoard();
        } else {
          _routes.sublist(newRoutes.length, _routes.length).forEach((route) {
            navigator.pop();
          });
        }
      }
      _routes.clear();
      _routes.addAll(newRoutes);
    });
  }
}

class OsamTabNavigator<P extends TabNavigationPresenter>
    extends StatelessWidget {
  final Map<String, Widget> pages;
  final Map<String, Widget> tabs;
  final BoxDecoration tabDecoration;
  final BoxConstraints tabBarConstraints;
  final bool handleOrientation;

  const OsamTabNavigator(
      {Key key,
      @required this.pages,
      this.tabDecoration,
      @required this.tabs,
      this.tabBarConstraints,
      this.handleOrientation})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    final presenter = context.presenter<P>();
    return ValueListenableBuilder(
        valueListenable: presenter.tabNavigation.tabNotifier,
        builder: (_, value, __) {
          final currentPage = pages[value];
          final currentPresenter =
              currentPage is PresenterProvider<NavigationPresenter>
                  ? currentPage.presenter
                  : null;
          final children = pages.keys
              .map((tab) => Expanded(
                    child: GestureDetector(
                      child: Container(
                        child: tabs[tab],
                        alignment: Alignment.center,
                        color: Colors.transparent,
                      ),
                      onTap: () {
                        if (value != tab) {
                          presenter.changeTab(tab);
                        } else {
                          final routes = currentPresenter?.currentRoutes ?? [];
                          if (routes.length > 1) {
                            currentPresenter.pushAndRemoveUntil(routes.first);
                          }
                        }
                      },
                    ),
                  ))
              .toList();
          return OrientationBuilder(builder: (context, orientation) {
            return Stack(
              children: <Widget>[
                PageTransitionSwitcher(
                  transitionBuilder: (
                    Widget child,
                    Animation<double> animation,
                    Animation<double> secondaryAnimation,
                  ) {
                    return FadeThroughTransition(
                      animation: animation,
                      secondaryAnimation: secondaryAnimation,
                      child: child,
                    );
                  },
                  child: currentPage,
                ),
                Positioned(
                  bottom: 0,
                  right: orientation == Orientation.portrait ? null : 0,
                  child: Container(
                    decoration: tabDecoration,
                    padding: const EdgeInsets.symmetric(vertical: 5),
                    constraints: orientation == Orientation.portrait
                        ? tabBarConstraints
                        : BoxConstraints(
                            maxWidth: tabBarConstraints.maxHeight,
                            maxHeight: MediaQuery.of(context).size.height),
                    child: orientation == Orientation.portrait
                        ? Row(
                            children: children,
                          )
                        : Column(children: children),
                  ),
                )
              ],
            );
          });
        });
  }
}

==> osam_flutter-3.3.6.tar.gz: lib/src/navigation/osam_navigator.dart <==
import 'package:after_layout/after_layout.dart';
import 'package:animations/animations.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:osam_flutter/src/navigation/osam_route.dart';
import '../../osam_flutter.dart';

typedef OsamDialog = Future Function(BuildContext context);

class OsamNavigator<P extends NavigationPresenter> extends StatefulWidget {
  final Map<Type, WidgetBuilder> routes;
  final Map<Type, OsamDialog> dialogs;
  final bool isMaterial;
  final bool log;
  final bool automaticHideKeyBoardOnPush;

  const OsamNavigator(
      {Key key,
      this.routes,
      this.isMaterial = false,
      this.log = false,
      this.automaticHideKeyBoardOnPush = false,
      this.dialogs})
      : super(key: key);

  @override
  _OsamNavigatorState createState() => _OsamNavigatorState<P>();
}

class _OsamNavigatorState<P extends NavigationPresenter>
    extends State<OsamNavigator<P>> with AfterLayoutMixin {
  final _routes = <OsamRoute<Object>>[];
  var lastKnownStackSize = 0;

  Map<Type, WidgetBuilder> get routes => widget.routes;

  Map<Type, OsamDialog> get dialogs => widget.dialogs;

  @override
  Widget build(BuildContext context) =>
      routes[context.presenter<P>().firstRoute.runtimeType](context);

  void silentPop(P presenter) {
    if (_routes.length > 1) {
      lastKnownStackSize = lastKnownStackSize - 1;
      if (_routes.length > lastKnownStackSize) {
        _routes.removeLast();
        presenter.pop();
      }
    }
  }

  void hideKeyBoard() =>
      SystemChannels.textInput.invokeMethod('TextInput.hide');

  PageRoute _pageRoute({WidgetBuilder widgetBuilder, RouteSettings settings}) =>
      widget.isMaterial
          ? MaterialPageRoute(builder: widgetBuilder, settings: settings)
          : CupertinoPageRoute(builder: widgetBuilder, settings: settings);

  @override
  void afterFirstLayout(BuildContext context) {
    final navigator = Navigator.of(context);
    final currentContext = navigator.overlay.context;
    final presenter = context.presenter<P>();
    final initialRoutes = presenter.currentRoutes;
    _routes.addAll(initialRoutes);
    if (initialRoutes.length > 1) {
      final others = initialRoutes.sublist(1, initialRoutes.length);
      others.forEach((route) {
        final target = routes[route.runtimeType] ?? dialogs[route.runtimeType];
        if (!(target is WidgetBuilder)) {
          (target(currentContext) as Future).then((_) {
            route.complete();
            silentPop(presenter);
          });
        } else {
          navigator.push(_pageRoute(widgetBuilder: target)).then((value) {
            route.complete();
            silentPop(presenter);
          });
        }
      });
      if (widget.automaticHideKeyBoardOnPush) hideKeyBoard();
    }
    lastKnownStackSize = _routes.length;
    presenter.navigation.routes.addListener(() {
      final newRoutes = presenter.currentRoutes;
      if (widget.log) {
        print('Last routes: $_routes');
        print('New routes: $newRoutes');
      }
      final routesDifferentNumber = newRoutes.length - _routes.length;
      if (routesDifferentNumber == 0 && newRoutes.last != _routes.last) {
        final newRoute = newRoutes.last;
        final target =
            routes[newRoute.runtimeType] ?? dialogs[newRoute.runtimeType];
        if (!(target is WidgetBuilder)) {
          (target(currentContext) as Future).then((_) {
            newRoute.complete();
            silentPop(presenter);
          });
        } else {
          navigator
              .pushReplacement(_pageRoute(widgetBuilder: target))
              .then((value) {
            newRoute.complete();
            silentPop(presenter);
          });
        }
        lastKnownStackSize = lastKnownStackSize + 1;
        if (widget.automaticHideKeyBoardOnPush) hideKeyBoard();
      } else if (routesDifferentNumber > 0) {
        final pushRoutes = newRoutes.sublist(_routes.length, newRoutes.length);
        pushRoutes.forEach((route) {
          final target =
              routes[route.runtimeType] ?? dialogs[route.runtimeType];
          if (!(target is WidgetBuilder)) {
            (target(currentContext) as Future).then((_) {
              route.complete();
              silentPop(presenter);
            });
          } else {
            navigator.push(_pageRoute(widgetBuilder: target)).then((_) {
              route.complete();
              silentPop(presenter);
            });
          }
        });
        if (widget.automaticHideKeyBoardOnPush) hideKeyBoard();
        lastKnownStackSize = newRoutes.length;
      } else if (routesDifferentNumber < 0) {
        if (_routes.first != newRoutes.first) {
          lastKnownStackSize = newRoutes.length;
          final newRoute = newRoutes.first;
          final target =
              routes[newRoute.runtimeType] ?? dialogs[newRoute.runtimeType];
          if (!(target is WidgetBuilder)) throw Error();
          navigator
              .pushAndRemoveUntil(
                  _pageRoute(widgetBuilder: target), (route) => false)
              .then((value) {
            newRoute.complete();
            silentPop(presenter);
          });
          if (widget.automaticHideKeyBoardOnPush) hideKeyBoard();
        } else {
          _routes.sublist(newRoutes.length, _routes.length).forEach((route) {
            navigator.pop();
          });
        }
      }
      _routes.clear();
      _routes.addAll(newRoutes);
    });
  }
}

class OsamTabNavigator<P extends TabNavigationPresenter>
    extends StatelessWidget {
  final Map<String, Widget> pages;
  final Map<String, Widget> tabs;
  final Map<String, Widget> selectedTabs;
  final BoxDecoration tabDecoration;
  final BoxConstraints tabBarConstraints;

  const OsamTabNavigator(
      {Key key,
      @required this.pages,
      this.tabDecoration,
      @required this.tabs,
      @required this.selectedTabs,
      @required this.tabBarConstraints})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    final presenter = context.presenter<P>();
    return ValueListenableBuilder<String>(
        valueListenable: presenter.tabNavigation.tabNotifier,
        builder: (_, value, __) {
          final currentPage = pages[value];
          final currentPresenter =
              currentPage is PresenterProvider<NavigationPresenter>
                  ? currentPage.presenter
                  : null;
          final children = pages.keys
              .map((tab) => Expanded(
                    child: GestureDetector(
                      child: Container(
                        child: AnimatedCrossFade(
                          duration: const Duration(milliseconds: 300),
                          firstChild: tabs[tab],
                          secondChild: selectedTabs[tab],
                          crossFadeState: value == tab
                              ? CrossFadeState.showSecond
                              : CrossFadeState.showFirst,
                        ),
                        alignment: Alignment.center,
                        color: Colors.transparent,
                      ),
                      onTap: () {
                        if (value != tab) {
                          presenter.changeTab(tab);
                        } else {
                          final routes = currentPresenter?.currentRoutes ?? [];
                          if (routes.length > 1) {
                            currentPresenter.pushAndRemoveUntil(routes.first);
                          }
                        }
                      },
                    ),
                  ))
              .toList();
          return OrientationBuilder(builder: (context, orientation) {
            return Stack(
              children: <Widget>[
                PageTransitionSwitcher(
                  transitionBuilder: (
                    Widget child,
                    Animation<double> animation,
                    Animation<double> secondaryAnimation,
                  ) {
                    return FadeThroughTransition(
                      animation: animation,
                      secondaryAnimation: secondaryAnimation,
                      child: child,
                    );
                  },
                  child: currentPage,
                ),
                Positioned(
                  bottom: 0,
                  right: orientation == Orientation.portrait ? null : 0,
                  child: Container(
                    decoration: tabDecoration,
                    padding: const EdgeInsets.symmetric(vertical: 5),
                    constraints: orientation == Orientation.portrait
                        ? tabBarConstraints
                        : BoxConstraints(
                            maxWidth: tabBarConstraints.maxHeight,
                            maxHeight: MediaQuery.of(context).size.height),
                    child: orientation == Orientation.portrait
                        ? Row(
                            children: children,
                          )
                        : Column(children: children),
                  ),
                )
              ],
            );
          });
        });
  }
}
